/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _COVER_THREAD_
#define _COVER_THREAD_

#include "coverprovider.h"

#include <QThread>
#include <QStringList>
#include <QObject>
#include <QByteArray>

/*
********************************************************************************
*                                                                              *
*    Class CoverThread                                                         *
*                                                                              *
********************************************************************************
*/   
class CoverThread : public QThread
{
Q_OBJECT
 public:
    explicit CoverThread(QObject *parent = 0);
    ~CoverThread();
    void setExit(bool b) {m_exit = b;}
    void setRequest(const QString& artist, const QString& album);

  protected:
    void run();

  private slots:
    void slot_check_database();
    void slot_process_cover_search();
    void slot_start_cover_provider();
    void slot_finish_cover_search();
    void slot_save_cover(int, QByteArray);
    
  private:
    struct RequestParam
    {
      RequestParam(){};
      
      QString artist;
      QString album;
      QString cover_path;      
    };
  
    QList<RequestParam>    m_list_param;
    RequestParam           m_current_param;    
    int                    m_current_request_id;
    int                    m_max;
    bool                   m_exit;
    
    //! perform search on all missing cover or on specific request
    bool                   m_isFullDbSearch;

    QList<CoverProvider*>  m_coverprovider_list;
    int                    m_current_provider_id;
    
  signals:
    void finished();
    void progress(int);
};


#endif // _COVER_THREAD_
