/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _PLSFILE_MODEL_H_
#define _PLSFILE_MODEL_H_

#include <QVariant>
#include <QString>
#include <QStringList>
#include <QObject>

#include "MediaItem/mediaitem.h"

/*
********************************************************************************
*                                                                              *
*    Class PlsFileModel                                                        *
*                                                                              *
********************************************************************************
*/
class PlsFileModel : public QObject
{
Q_OBJECT
     static PlsFileModel*  INSTANCE;

  public:
     PlsFileModel(QObject *parent = 0);
     ~PlsFileModel();
     static PlsFileModel* instance() { return INSTANCE; }    
    
     //! custom method
     MediaItemPtr rootItem();
     MediaItemPtr addChild(const MediaItemPtr  parent, T_TYPE t);
     MediaItemPtr getItem(const MediaItemPtr parent,const int childIdx); 
     void clear();
     
     //! get Url method
     QList<MediaItemPtr> getItemChildrenUrls(const MediaItemPtr parent);

     //! filtering method
     bool isUrlFiltered(const MediaItemPtr urlItem);
     bool isPlaylistFiltered(const MediaItemPtr playlistItem);
     void setFilter(const QString & f) {m_filter_pattern = f;}
     
     //! playing status method
     bool itemIsPlaying(const MediaItemPtr item);
     void setItemIsPlaying(const MediaItemPtr item, bool playing);
     
     MediaItemPtr getPlayingUrlItem();
     void updatePlayingItem(const QString& url);
     void stopPlayingItem();

     //! Favorites status method
     bool itemIsFavorite(const MediaItemPtr item);
     void setItemIsFavorite(const MediaItemPtr item, bool isfavorite);
     
     //! Broken status method
     void updateBrokenItem(const QString& url);
     
  signals:
    void signalPlayingStatusChanged();
    void signalFavoriteStatusChanged();    
    void modelCleared();

  private:
     MediaItemPtr           m_rootItem;
     int                    m_playing_item_idx;
     QString                m_filter_pattern;
};

#endif // _PLSFILE_MODEL_H_
