/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/plsfilepopulator.h"
#include "CollectionModel/plsfilemodel.h"
#include "MediaItem/mediaitem.h"
#include "Database/database.h"

#include "EnginePlayer/engineplayer.h"
#include "debug.h"


#include <QFileInfo>
#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>

/*
********************************************************************************
*                                                                              *
*    Class PlsFilePopulator                                                    *
*                                                                              *
********************************************************************************
*/

PlsFilePopulator::PlsFilePopulator()
{
    _plsFileModel    = PlsFileModel::instance();
    _exit            = false;
}

void PlsFilePopulator::run()
{
    Debug::debug() << " --- PlsFilePopulator --> Starting model population";
    /*-----------------------------------------------------------*/
    /* Get connection                                            */
    /* ----------------------------------------------------------*/    
    Database db;
    if (!db.connect("PlsFilePopulator")) {
      emit populatingFinished();
      return;
    }

    /*-----------------------------------------------------------*/
    /* Get file count from database                              */
    /* ----------------------------------------------------------*/
    QSqlQuery queryCount("SELECT COUNT(*) FROM `playlist_items`",*db.sqlDb());
    queryCount.next();
  
    int _progressMax = queryCount.value(0).toInt();
    int _progress    = 0;       
    
    /*-----------------------------------------------------------*/
    /* Parse Database for normal playlist and populate model     */
    /* ----------------------------------------------------------*/
    _plsFileModel->clear();
    
    MediaItemPtr playlistItem = MediaItemPtr(0);
    MediaItemPtr trackItem    = MediaItemPtr(0);

    QVariant playlist_id = -1;
    QVariant item_id     = -1;
    

    QSqlQuery query("SELECT playlist_id,playlist_filename,playlist_name,playlist_type,playlist_favorite,playlist_mtime, \
                            id,url,name, \
                            track_trackname,number,length,playcount,albumgain,albumpeakgain,trackgain,trackpeakgain, \
                            artist_name,album_name,genre_name,year \
                     FROM view_playlists \
                     ORDER BY playlist_type ASC",*db.sqlDb());                              
    
    while (query.next() && !_exit) 
    {
        if(  playlist_id != query.value(0) ) {
          playlist_id = query.value(0);
          
          playlistItem = _plsFileModel->addChild(_plsFileModel->rootItem(), TYPE_PLAYLIST);
          playlistItem->data[PLAYLIST::id]         =  query.value(0);
          playlistItem->data[PLAYLIST::url]        =  query.value(1);
          playlistItem->data[PLAYLIST::name]       =  query.value(2);
          playlistItem->data[PLAYLIST::icon]       =  QString(":/images/media-playlist-110x110.png");
          playlistItem->data[PLAYLIST::type]       =  query.value(3);
          playlistItem->data[PLAYLIST::isFavorite] =  query.value(4);
          playlistItem->data[PLAYLIST::date]       =  query.value(5);
          playlistItem->setParent(_plsFileModel->rootItem());  
        }
        
        if(  item_id != query.value(6) ) {
          //! new playlist item  
          item_id = query.value(6);

          if(!query.value(9).isNull()) 
          {
            trackItem = _plsFileModel->addChild(playlistItem, TYPE_TRACK);

            //media->data[TRACK::id]         = query.value(0);
            trackItem->data[TRACK::url]        = query.value(7);
            trackItem->data[TRACK::name]       = query.value(7);
            trackItem->data[TRACK::title]      = query.value(9);
            trackItem->data[TRACK::num]        = query.value(10);
            trackItem->data[TRACK::duration]   = query.value(11);
            trackItem->data[TRACK::playcount]  = query.value(12);
            trackItem->data[TRACK::artist]     = query.value(17);
            trackItem->data[TRACK::album]      = query.value(18);
            trackItem->data[TRACK::genre]      = query.value(19);
            trackItem->data[TRACK::year]       = query.value(20);
            trackItem->data[TRACK::albumGain]  = query.value(13);
            trackItem->data[TRACK::albumPeak]  = query.value(14);
            trackItem->data[TRACK::trackGain]  = query.value(15);
            trackItem->data[TRACK::trackPeak]  = query.value(16);
          }
          else 
          {
            //! playlist_item is NOT in collection
            if(MEDIA::isUrlLocal(query.value(7).toString())) 
            {
              trackItem = _plsFileModel->addChild(playlistItem, TYPE_TRACK);

              MediaItemPtr media = MEDIA::FromLocalFile( query.value(7).toString());
              if(media) {
                trackItem->data = media->data;
                media.reset();
              }
              else {
                trackItem->data[TRACK::id]       =  -1;
                trackItem->data[TRACK::url]      =  query.value(7);
                trackItem->data[TRACK::title]     = query.value(8);
              }
            }
            else 
            {
              trackItem = _plsFileModel->addChild(playlistItem, TYPE_STREAM);              
              trackItem->data[STREAM::id]      =  -1;
              trackItem->data[STREAM::url]     =  query.value(7);
              trackItem->data[STREAM::name]    =  query.value(8);     
            }
          }
          trackItem->setParent(playlistItem);

          _progress++;
        } // fin if new playlist item

        if( _progressMax != 0)  // when only smart playlist exist
          emit populatingProgress(  (int) ( (_progress*100)/_progressMax ) );
        //Debug::debug() << " --- PlsFilePopulator -->populatingProgress :" << percent << " %";

    } //! Fin while query
        
    
    /*-----------------------------------------------------------*/
    /* Parse Smart playlist and populate model                   */
    /* ----------------------------------------------------------*/
    QSqlQuery query_2("SELECT id,name,icon,rules,type,favorite FROM smart_playlists ORDER BY type ASC",*db.sqlDb());
    
    while (query_2.next() && !_exit) {
      playlistItem = _plsFileModel->addChild(_plsFileModel->rootItem(), TYPE_PLAYLIST);
      playlistItem->data[PLAYLIST::id]         =  query_2.value(0);
      playlistItem->data[PLAYLIST::name]       =  query_2.value(1);
      playlistItem->data[PLAYLIST::icon]       =  query_2.value(2);
      playlistItem->data[PLAYLIST::rules]      =  query_2.value(3);
      playlistItem->data[PLAYLIST::type]       =  query_2.value(4);
      playlistItem->data[PLAYLIST::isFavorite] =  query_2.value(5);
      playlistItem->setParent(_plsFileModel->rootItem());        
    }    
    
    /*-----------------------------------------------------------*/
    /* Update now playing status into model                      */
    /* ----------------------------------------------------------*/
    QString currentTrack = EnginePlayer::instance()->enginePlayingUrl();
    if(!currentTrack.isEmpty()) {
      _plsFileModel->updatePlayingItem(currentTrack);
    }
    
    /*-----------------------------------------------------------*/
    /* End                                                       */
    /* ----------------------------------------------------------*/ 
    Debug::debug() << " --- PlsFilePopulator --> End model population";
    if(!_exit)
      emit populatingFinished();
}
