/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionView/collectionscene.h"

#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/plsfilemodel.h"
#include "Playlist/playlistmodel.h"
#include "MediaItem/mediaitem.h"
#include "SmartPlaylist/smartplaylist.h" 

#include "threadmanager.h"
#include "debug.h"

/*
********************************************************************************
*                                                                              *
*    Class CollectionScene                                                     *
*        -> playqueue method                                                   *
********************************************************************************
*/ 
void CollectionScene::enqueueTrack(bool is_new_playqueue)
{
    //Debug::debug() << " ---- enqueueTrack";
    if(!_mouseGrabbedItem) return;
    TrackGraphicItem *item = static_cast<TrackGraphicItem*>(_mouseGrabbedItem);
  
    if(item->media) {
      if(is_new_playqueue)
        _playlistModel->clear();
      
      _playlistModel->addMediaItem(item->media);
    }
}

    
void CollectionScene::enqueueAlbum(bool is_new_playqueue) 
{
    //Debug::debug() << " ---- enqueueAlbum";
    if(!_mouseGrabbedItem) return;
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);
  
    QList<MediaItemPtr> list = _collectionModel->getItemChildrenTracks(item->media);

    if(!list.isEmpty()) {
      if(is_new_playqueue)
        _playlistModel->clear();

      ThreadManager::instance()->playlistAddMediaItems(list);
    }
}

void CollectionScene::enqueueAlbumGenre(bool is_new_playqueue)
{
    //Debug::debug() << " ---- enqueueAlbumGenre";
    if(!_mouseGrabbedItem) return;
    AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem*>(_mouseGrabbedItem);
  
    QList<MediaItemPtr> list = _collectionModel->getAlbumChildrenTracksGenre(item->media,item->_genre);

    if(!list.isEmpty()) {
      if(is_new_playqueue)
        _playlistModel->clear();

      ThreadManager::instance()->playlistAddMediaItems(list);
    }
}

void CollectionScene::enqueueArtist(bool is_new_playqueue)
{
    //Debug::debug() << " ---- enqueueArtist";
    if(!_mouseGrabbedItem) return;
    ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);
  
    QList<MediaItemPtr> list = _collectionModel->getItemChildrenTracks(item->media);

    if(!list.isEmpty()) {
      if(is_new_playqueue)
        _playlistModel->clear();

      ThreadManager::instance()->playlistAddMediaItems(list);
    }
}



void CollectionScene::enqueuePlaylist(bool is_new_playqueue)
{
    if(!_mouseGrabbedItem) return;
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);
    
    QList<MediaItemPtr> list;
    
    //! rechercher dynamique pour les smart playlists
    if(item->media->data.at(PLAYLIST::type).toInt() == T_SMART)
    {
      list = SmartPlaylist::mediaItem( item->media->data.at(PLAYLIST::rules) );
    }
    //! playliste normale -> lecture dans le modele
    else {
      list = _plsFileModel->getItemChildrenUrls(item->media);
    }
    
    if(!list.isEmpty()) {
      if(is_new_playqueue)
        _playlistModel->clear();
      
      ThreadManager::instance()->playlistAddMediaItems(list);
    }
}