/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//! LOCAL
#include "provider_lastfm.h"
#include "networkaccess.h"
#include "utilities.h"
#include "constants.h"
#include "debug.h"

/*
********************************************************************************
*                                                                              *
*    Class LastFmProvider                                                      *
*                                                                              *
********************************************************************************
*/ 
LastFmProvider::LastFmProvider(QObject* parent)  : InfoProvider("lastfm")
{
}

LastFmProvider::~LastFmProvider()
{
    cancel_searches();
}
/*******************************************************************************
  LastFmProvider::init
*******************************************************************************/ 
void LastFmProvider::init() 
{
}

/*******************************************************************************
  LastFmProvider::cancel_searches
*******************************************************************************/ 
void LastFmProvider::cancel_searches() 
{
    Debug::debug() << "LastFmProvider: cancel_searches";
    
    foreach (QObject* object, m_requests.keys()) {
      m_requests.take(object);
      delete object;
    }
      
    m_requests.clear();
}

/*******************************************************************************
  LastFmProvider::fetch_info
*******************************************************************************/ 
void LastFmProvider::fetch_info(int id) 
{
    if( !is_enabled() ) return;
    
    fetch_album_info(id);
}


/*******************************************************************************
  LastFmProvider::fetch_album_info
*******************************************************************************/
void LastFmProvider::fetch_album_info(int id)
{
    // we genere the url for the demand on the lastFM Api
    QUrl url("http://ws.audioscrobbler.com/2.0/");
    url.addQueryItem("method", "album.getinfo");
    url.addQueryItem("api_key", LASTFM_API_KEY);
    url.addQueryItem("artist", m_current_media->data.at(TRACK::artist).toString().toLocal8Bit());
    url.addQueryItem("album", m_current_media->data.at(TRACK::album).toString().toLocal8Bit());

    QObject *reply = HTTP()->get(url);
    m_requests[reply] = id;
    connect(reply, SIGNAL(data(QByteArray)), SLOT(slot_fetch_album_info_done(QByteArray)));
}


/*******************************************************************************
  LastFmProvider::slot_fetch_album_info_done
*******************************************************************************/
void LastFmProvider::slot_fetch_album_info_done(QByteArray bytes) 
{
    Debug::debug() << " ---- LastFmProvider::slot_fetch_album_info_done";

    QObject* reply = qobject_cast<QObject*>(sender());
    if (!reply || !m_requests.contains(reply))   return;
    
    const int id = m_requests.take(reply);
    reply->deleteLater();
        
    //! xml reader
    QXmlStreamReader xml(bytes);
    bool imageOK = false;
    bool bioOK   = false;

    while(!xml.atEnd() && !xml.hasError()) {
      QXmlStreamReader::TokenType token = xml.readNext();

      if(token == QXmlStreamReader::StartElement) {
            //! get album image
            if(!imageOK && xml.name() == "image" && xml.attributes().value("size") == "extralarge") 
            {
              imageOK = true;
              QString imageUrl = xml.readElementText();
              if (!imageUrl.isEmpty()) {
                QUrl url = QUrl::fromEncoded(imageUrl.toUtf8());
                QObject *reply = HTTP()->get(url);
                m_requests[reply] = id;
                connect(reply, SIGNAL(data(QByteArray)), SLOT(slot_album_image_done(QByteArray)));
              }
            }
            //! get album bio
            else if(!bioOK && xml.name() == "content") {
                bioOK = true;
                QString bio = xml.readElementText();
                //Debug::debug() << " ---- InfoDownloader::bio found" << bio;
                if (!bio.isEmpty()) {
                  emit AlbumBioOK(id,bio);
                }
            }
      }
      if (imageOK && bioOK) break;
    }
}


/*******************************************************************************
  LastFmProvider::slot_album_image_done
*******************************************************************************/
void LastFmProvider::slot_album_image_done(QByteArray bytes)
{
    QObject* reply = qobject_cast<QObject*>(sender());
    if (!reply || !m_requests.contains(reply)) return;
    
    const int id = m_requests.take(reply);
    reply->deleteLater();
    
    emit AlbumImageOK(id,bytes);
}

