/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "statuswidget.h"
#include "mainwindow.h"
//#include "debug.h"

#include <QtGui>

/*
********************************************************************************
*                                                                              *
*    Class StatusWidget                                                        *
*                                                                              *
********************************************************************************
*/   

StatusWidget* StatusWidget::INSTANCE = 0;


StatusWidget::StatusWidget(QWidget *parent) : QWidget(parent)
{
    INSTANCE = this;
    _parent = parent;
        
    setAttribute(Qt::WA_TransparentForMouseEvents);
    setAttribute(Qt::WA_NoSystemBackground);
    setWindowFlags(Qt::FramelessWindowHint | Qt::X11BypassWindowManagerHint);

    
    QPalette palette = QToolTip::palette();
    setPalette(palette);
    
    setWindowOpacity(style()->styleHint(QStyle::SH_ToolTipLabel_Opacity, 0, this) / 255.0);

    //! label
    message = new QLabel(this);
    message->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Preferred);

    
    message->setPalette(palette);
    message->setForegroundRole(QPalette::ToolTipText);
     
    message->setFont(QApplication::font("QTipLabel"));
    message->setMargin(0);
    message->setIndent(1);  
    message->setAlignment(Qt::AlignLeft);
    message->setFrameStyle(QFrame::NoFrame);    
    
    //! pixmap
    pixmap  = new QLabel(this);
    pixmap->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum);    
    
    
    //! layout
    QHBoxLayout* layout = new QHBoxLayout;
    layout->setContentsMargins (4,4,4,4);

    layout->addWidget(pixmap);
    layout->addWidget(message);
    setLayout(layout);
 
    setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Preferred);
    isShortMessage = false;
}








static uint messageId = 0;

//! --------- startProgressMessage ---------------------------------------------
uint StatusWidget::startProgressMessage(const QString& action)
{
     //Debug::debug() << "- StatusWidget -> startProgressMessage";

    _longMessage.insert(++messageId, action);
    
    updateMessage();
    
    return messageId;
}

//! --------- stopProgressMessage ----------------------------------------------
void StatusWidget::stopProgressMessage(uint id)
{
    //Debug::debug() << "- StatusWidget -> stopProgressMessage -> id "<< id;
    if(_longMessage.contains(id))
      _longMessage.remove(id);
    
    updateMessage();
}

//! --------- updateProgressMessage --------------------------------------------
void StatusWidget::updateProgressMessage(uint id, const QString & message)
{
    if(_longMessage.contains(id))
      _longMessage[id] = message;
    
    updateMessage();
}

//! --------- startShortMessage ------------------------------------------------
void StatusWidget::startShortMessage(const QString& m, STATUS::T_MESSAGE type, int ms )
{
    //Debug::debug() << "- StatusWidget -> startShortMessage ->"<< m;

    //! only one short message at one time
    if(isShortMessage) 
      stopShortMessage();
    isShortMessage = true;
        
    message->setText(m);
    switch(type) {
      case STATUS::TYPE_INFO       : pixmap->setPixmap(QPixmap(":/images/info-18x18.png")); break;
      case STATUS::TYPE_WARNING    : pixmap->setPixmap(QPixmap(":/images/warning-18x18.png"));break;
      case STATUS::TYPE_ERROR      : pixmap->setPixmap(QPixmap(":/images/media-broken-18x18.png"));break;
      default : pixmap->setPixmap(QPixmap());break;
    }     
    
    QTimer::singleShot(ms, this, SLOT(stopShortMessage()));

    updateMessage();
}

//! --------- stopShortMessage -------------------------------------------------
void StatusWidget::stopShortMessage()
{
    isShortMessage = false;

    updateMessage();
}


//! --------- updateMessage ----------------------------------------------------
void StatusWidget::updateMessage()
{
    if(isShortMessage) {    
      resize( sizeHint() );
      setPosition();
      update();
      this->show();
      return;
    }
    
    if(_longMessage.isEmpty()) { 
      this->hide();
    }
    else {
      QStringList list = _longMessage.values();
      message->setText(list.first());
      pixmap->setPixmap(QPixmap(":/images/info-18x18.png.png"));
      this->show();
    }    
    
    resize( sizeHint() );
    setPosition();
    update();     
}



void StatusWidget::setPosition() 
{
    //Debug::debug() << "- StatusWidget -> setPosition   _parent->height() "  << _parent->height();
    move( 2, _parent->height() - this->height()-2);
}


void StatusWidget::moveWidget()
{
    setPosition();
}

void StatusWidget::moveEvent ( QMoveEvent * event )
{
Q_UNUSED(event)
  if(isVisible())
     setPosition();
}

void StatusWidget::resizeEvent ( QResizeEvent * event )
{
Q_UNUSED(event)
  if(isVisible())
    setPosition();
}    



QSize StatusWidget::sizeHint() const
{
    QFontMetrics fm( QApplication::font("QTipLabel") );
    int x = fm.width(message->text(),-1) + pixmap->width() + 20;
    int y = fm.height() + 12;
    return QSize(x, y);
}

void StatusWidget::paintEvent(QPaintEvent *ev)
{
    // QToolTip paint event
    QStylePainter p(this);
    QStyleOptionFrame opt;
    opt.init(this);
    
    p.drawPrimitive(QStyle::PE_PanelTipLabel, opt);
    p.end();

    QWidget::paintEvent(ev);   
}
