/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _NETWORKACCESS_H_
#define _NETWORKACCESS_H_

#include <QtNetwork>
#include <QHash>
#include <QThread>

#include "constants.h"

/*
********************************************************************************
*                                                                              *
*    Class NetworkReply                                                        *
*                                                                              *
********************************************************************************
*/                   
class NetworkReply : public QObject 
{
    Q_OBJECT

public:
    NetworkReply(QNetworkReply* networkReply);
    bool followRedirects;

public slots:
    void finished();
    void requestError(QNetworkReply::NetworkError);

signals:
    void data(QByteArray);
    void error(QNetworkReply*);
    void finished(QNetworkReply*);

private slots:
    void abort();

private:
    QNetworkReply *networkReply;
    QTimer *timer;

};

/*
********************************************************************************
*                                                                              *
*    Class NetworkAccess                                                       *
*                                                                              *
********************************************************************************
*/                   
class NetworkAccess : public QObject 
{
    Q_OBJECT

public:
    NetworkAccess(QObject* parent=0);
    QNetworkReply* simpleGet(QUrl url, int operation = QNetworkAccessManager::GetOperation);
    NetworkReply* get(QUrl url);
    NetworkReply* head(QUrl url);
    QNetworkReply* syncGet(QUrl url);
    QNetworkReply* syncHead(QUrl url);
    QByteArray syncGetBytes(QUrl url);
    QString syncGetString(QUrl url);
    bool syncTest(QUrl url);

private slots:
    void error(QNetworkReply::NetworkError);
    void syncMetaDataChanged();
    void syncFinished();

private:
    QNetworkReply *networkReply;
    bool working;
};

/*
********************************************************************************
*                                                                              *
*    Class NetworkManager                                                      *
*                                                                              *
********************************************************************************
*/                   
#define HTTP() (NetworkManager::instance()->http())

class NetworkManager
{
  static NetworkManager         *INSTANCE;
    
public:
  NetworkManager();

  static NetworkManager* instance() { return INSTANCE; }
  
  NetworkAccess* http();
  
private :
  QHash<QThread*, NetworkAccess *> m_http;
};



typedef QPointer<QObject> ObjectPointer;
Q_DECLARE_METATYPE(ObjectPointer)

#endif // _NETWORKACCESS_H_
